// Toolkit.h
// Neuron Computing Inc.

#ifndef __Toolkit_h__
#define __Toolkit_h__

#define MOTIF
#include "XWindows.h"

typedef unsigned long dword;

#define RGB(r, g, b) ((r)|((g)<<8)|((b)<<16))

// Methods
void messageBox(char* text);

// Forward references
class XApp;
class Component;
class Container;
class Frame;
class Graphics;

class Menu;
class MenuBar;

//////////////////////////////////////////////////////////////
// Point
class Point
{
public:
	int x;
	int y;
};

//////////////////////////////////////////////////////////////
// Rectangle
class Rectangle
{
public:
	int x;
	int y;
	int width;
	int height;
};

//////////////////////////////////////////////////////////////
// DimensionOb
class DimensionOb
{
public:
	int width;
	int height;
};

//////////////////////////////////////////////////////////////
// FontMetrics
class FontMetrics
{
public:
    int getHeight()
		{ return 13; }
    int stringWidth(char* str)
		{ return 7 * strlen(str); }
};

//////////////////////////////////////////////////////////////
// Color
class Color
{
public:
	dword m_rgb;
	Color()
		{m_rgb = 0;}
	Color(dword rgb)
		{m_rgb = rgb;}
	Color(int r, int g, int b)
		{m_rgb = RGB(r, g, b);}
};

//////////////////////////////////////////////////////////////
// Graphics
class Graphics
{
public:
	Widget m_w;
	GC m_gc;
public:
	Graphics();
	~Graphics();
public:
	FontMetrics* getFontMetrics();
	void setColor(Color c);
	// Drawing
    void drawLine(int x1, int y1, int x2, int y2);
    void drawString(int x, int y, char *str);
    void drawRect(int x, int y, int width, int height);
    void fillRect(int x, int y, int width, int height);
	void scroll(int x, int y, int width, int height, int dx, int dy);
};

//////////////////////////////////////////////////////////////
// Component
class Component
{
public:
	Widget m_w;
	Graphics* m_g;
public:
	Component();
	virtual ~Component();

	void registerEventHandlers(Widget w);

	GC getGC(int type);
	Graphics* getGraphics(Widget w);
// Properties
public:
	Rectangle* getBounds();
	DimensionOb* getSize();
	int getWidth();
	int getHeight();
	void setBounds(int x, int y, int width, int height);
	Color getBackground();
	void setBackground(Color c);
	Color getForeground();
	void setForeground(Color rgb);
	Font getFont();
	void setFont(Font font);
// Events
public:
	// Drawing
	virtual Graphics* getGraphics();

	void repaint(int time, int x, int y, int width, int height);
	void repaint(int time);

	virtual void update(Graphics* g);
	virtual void paint(Graphics* g) {}
	// Mouse
	virtual void mouseDown(int x, int y) {}
	virtual void mouseUp(int x, int y) {}
	virtual void mouseMove(int x, int y) {}
	// Menu
	virtual void actionPerformed(int id) {}
};

//////////////////////////////////////////////////////////////////
// Container
class Container : public Component
{
public:
	Widget m_layout;
public:
	Container();
	~Container();
};

//////////////////////////////////////////////////////////////////
// WindowOb
class WindowOb : public Container
{
public:
	WindowOb() {}
	~WindowOb() {}
};

//////////////////////////////////////////////////////////////////
// Frame
class Frame : public WindowOb
{
public:
	Widget m_workWindow;
	// Menu
	int m_menu_item_count;
	char** m_menu_items;
	char* m_menu_title;
	MenuBar* m_menu;
public:
	Frame();
	~Frame();
	void attach(Widget parent);
	void createLayout();

	Graphics* getGraphics();
	void setMenuBar(char* menu_title, char** items, int num_items);
};

/////////////////////////////////////////////////////////////
// Menu
class Menu
{
public:
	Widget m_menu;
public:
	Menu();
	void create(Widget menubar, char* name);
	void addMenuItems(char** menu_items, int menu_item_count);
};

/////////////////////////////////////////////////////////////
// MenuBar
class MenuBar
{
public:
	Widget m_menu;
public:
	MenuBar();
	void create(Frame* frame);
	void addMenu(char* name, char** menu_items, int menu_item_count);
};

//////////////////////////////////////////////////////////////
// XApp
class XApp
{
public:
	XtAppContext m_appContext;
	Display* m_display;
	Widget m_shell;
	Frame* m_mainFrame;
	int m_popup_active;

public:
	XApp();
	virtual ~XApp();

	void setDisplayArg(int& argc, char**& argv);
	Widget initApplication(int argc, char **argv);
	void init(int argc, char **argv);

	void setMainFrame(Frame* frame);
	void createMenu(char* menu_title, char** items, int num_items);
	void run();

	void eventLoop();
	void exitEventLoop();
};

//////////////////////////////////////////////////////////////////
// Dialog
class Dialog : public WindowOb
{
public:
	Bool m_done;
public:
	Dialog() {}
	~Dialog() {}

	void doModal();
};

/////////////////////////////////////////////////////////////
// FileDialog
class FileDialog : public Dialog
{
public:
	char m_fileName[256];
public:
	FileDialog();
	~FileDialog();

	void create(Widget parent);
};

#endif
